@@ -24,13 +24,29 @@ module Capistrano
       class GatewayConnectionFactory #:nodoc:
         def initialize(gateway, options)
           @options = options
-          @options[:logger].debug "Creating gateway using #{[*gateway].join(', ')}" if @options[:logger]
           Thread.abort_on_exception = true
-          @gateways = [*gateway].collect { |g| ServerDefinition.new(g) }
-          tunnel = SSH.connection_strategy(@gateways[0], @options) do |host, user, connect_options|
+          @gateways = {}
+          if gateway.is_a?(Hash)
+            @options[:logger].debug "Creating multiple gateways using #{gateway.inspect}" if @options[:logger]
+            gateway.each do |gw, hosts|
+              gateway_connection = add_gateway(gw)
+              [*hosts].each do |host|
+                @gateways[:default] ||= gateway_connection
+                @gateways[host] = gateway_connection
+              end
+            end
+          else
+            @options[:logger].debug "Creating gateway using #{[*gateway].join(', ')}" if @options[:logger]
+            @gateways[:default] = add_gateway(gateway)
+          end
+        end
+
+        def add_gateway(gateway)
+          gateways = [*gateway].collect { |g| ServerDefinition.new(g) }
+          tunnel = SSH.connection_strategy(gateways[0], @options) do |host, user, connect_options|
             Net::SSH::Gateway.new(host, user, connect_options)
           end
-          @gateway = (@gateways[1..-1]).inject(tunnel) do |tunnel, destination|
+          (gateways[1..-1]).inject(tunnel) do |tunnel, destination|
             @options[:logger].debug "Creating tunnel to #{destination}" if @options[:logger]
             local_host = ServerDefinition.new("127.0.0.1", :user => destination.user, :port => tunnel.open(destination.host, (destination.port || 22)))
             SSH.connection_strategy(local_host, @options) do |host, user, connect_options|
@@ -41,11 +57,15 @@ module Capistrano
         
         def connect_to(server)
           @options[:logger].debug "establishing connection to `#{server}' via gateway" if @options[:logger]
-          local_host = ServerDefinition.new("127.0.0.1", :user => server.user, :port => @gateway.open(server.host, server.port || 22))
+          local_host = ServerDefinition.new("127.0.0.1", :user => server.user, :port => gateway_for(server).open(server.host, server.port || 22))
           session = SSH.connect(local_host, @options)
           session.xserver = server
           session
         end
+
+        def gateway_for(server)
+          @gateways[server.host] || @gateways[:default]
+        end
       end
 
       # A hash of the SSH sessions that are currently open and available.
@@ -87,7 +107,7 @@ module Capistrano
       def connection_factory
         @connection_factory ||= begin
           if exists?(:gateway)
-            logger.debug "establishing connection to gateway `#{fetch(:gateway)}'"
+            logger.debug "establishing connection to gateway `#{fetch(:gateway).inspect}'"
             GatewayConnectionFactory.new(fetch(:gateway), self)
           else
             DefaultConnectionFactory.new(self)
@@ -118,8 +138,11 @@ module Capistrano
       # Destroys sessions for each server in the list.
       def teardown_connections_to(servers)
         servers.each do |server|
-          sessions[server].close
-          sessions.delete(server)
+          begin
+            sessions.delete(server).close
+          rescue IOError
+            # the TCP connection is already dead
+          end
         end
       end
 
@@ -133,7 +156,7 @@ module Capistrano
           servers = find_servers_for_task(task, options)
 
           if servers.empty?
-            if ENV['HOSTFILTER']
+            if ENV['HOSTFILTER'] || task.options.merge(options)[:on_no_matching_servers] == :continue
               logger.info "skipping `#{task.fully_qualified_name}' because no servers matched"
               return
             else
@@ -147,7 +170,10 @@ module Capistrano
           end
         else
           servers = find_servers(options)
-          raise Capistrano::NoMatchingServersError, "no servers found to match #{options.inspect}" if servers.empty?
+          if servers.empty?
+            raise Capistrano::NoMatchingServersError, "no servers found to match #{options.inspect}" if options[:on_no_matching_servers] != :continue
+            return
+          end
         end
 
         servers = [servers.first] if options[:once]
