@@ -32,6 +32,7 @@ _cset :rails_env, "production"
 _cset :rake, "rake"
 
 _cset :maintenance_basename, "maintenance"
+_cset(:maintenance_template_path) { File.join(File.dirname(__FILE__), "templates", "maintenance.rhtml") }
 
 # =========================================================================
 # These variables should NOT be changed unless you are very confident in
@@ -49,7 +50,7 @@ _cset(:release_name)      { set :deploy_timestamped, true; Time.now.utc.strftime
 
 _cset :version_dir,       "releases"
 _cset :shared_dir,        "shared"
-_cset :shared_children,   %w(system log pids)
+_cset :shared_children,   %w(public/system log tmp/pids)
 _cset :current_dir,       "current"
 
 _cset(:releases_path)     { File.join(deploy_to, version_dir) }
@@ -184,7 +185,7 @@ namespace :deploy do
   DESC
   task :setup, :except => { :no_release => true } do
     dirs = [deploy_to, releases_path, shared_path]
-    dirs += shared_children.map { |d| File.join(shared_path, d) }
+    dirs += shared_children.map { |d| File.join(shared_path, d.split('/').last) }
     run "#{try_sudo} mkdir -p #{dirs.join(' ')}"
     run "#{try_sudo} chmod g+w #{dirs.join(' ')}" if fetch(:group_writable, true)
   end
@@ -200,7 +201,7 @@ namespace :deploy do
   task :update do
     transaction do
       update_code
-      symlink
+      create_symlink
     end
   end
 
@@ -246,11 +247,11 @@ namespace :deploy do
     run <<-CMD
       rm -rf #{latest_release}/log #{latest_release}/public/system #{latest_release}/tmp/pids &&
       mkdir -p #{latest_release}/public &&
-      mkdir -p #{latest_release}/tmp &&
-      ln -s #{shared_path}/log #{latest_release}/log &&
-      ln -s #{shared_path}/system #{latest_release}/public/system &&
-      ln -s #{shared_path}/pids #{latest_release}/tmp/pids
+      mkdir -p #{latest_release}/tmp
     CMD
+    shared_children.map do |d|
+      run "ln -s #{shared_path}/#{d.split('/').last} #{latest_release}/#{d}"
+    end
 
     if fetch(:normalize_asset_timestamps, true)
       stamp = Time.now.utc.strftime("%Y%m%d%H%M.%S")
@@ -259,6 +260,14 @@ namespace :deploy do
     end
   end
 
+  desc <<-DESC
+    Deprecated API. This has become deploy:create_symlink, please update your recipes
+  DESC
+  task :symlink, :except => { :no_release => true } do
+    Kernel.warn "[Deprecation Warning] This API has changed, please hook `deploy:create_symlink` instead of `deploy:symlink`."
+    create_symlink
+  end
+
   desc <<-DESC
     Updates the symlink to the most recently deployed version. Capistrano works \
     by putting each new release of your application in its own directory. When \
@@ -268,7 +277,7 @@ namespace :deploy do
     deploy, including `restart') or the 'update' task (which does everything \
     except `restart').
   DESC
-  task :symlink, :except => { :no_release => true } do
+  task :create_symlink, :except => { :no_release => true } do
     on_rollback do
       if previous_release
         run "rm -f #{current_path}; ln -s #{previous_release} #{current_path}; true"
@@ -375,6 +384,8 @@ namespace :deploy do
       set :migrate_target, :latest
   DESC
   task :migrate, :roles => :db, :only => { :primary => true } do
+    rake = fetch(:rake, "rake")
+    rails_env = fetch(:rails_env, "production")
     migrate_env = fetch(:migrate_env, "")
     migrate_target = fetch(:migrate_target, :latest)
 
@@ -398,7 +409,7 @@ namespace :deploy do
     set :migrate_target, :latest
     update_code
     migrate
-    symlink
+    create_symlink
     restart
   end
 
@@ -411,12 +422,12 @@ namespace :deploy do
   DESC
   task :cleanup, :except => { :no_release => true } do
     count = fetch(:keep_releases, 5).to_i
-    if count >= releases.length
+    local_releases = capture("ls -xt #{releases_path}").split.reverse
+    if count >= local_releases.length
       logger.important "no old releases to clean up"
     else
-      logger.info "keeping #{count} of #{releases.length} deployed releases"
-
-      directories = (releases - releases.last(count)).map { |release|
+      logger.info "keeping #{count} of #{local_releases.length} deployed releases"
+      directories = (local_releases - local_releases.last(count)).map { |release|
         File.join(releases_path, release) }.join(" ")
 
       try_sudo "rm -rf #{directories}"
@@ -527,15 +538,19 @@ namespace :deploy do
               REASON="hardware upgrade" \\
               UNTIL="12pm Central Time"
 
+      You can use a different template for the maintenance page by setting the \
+      :maintenance_template_path variable in your deploy.rb file. The template file \ 
+      should either be a plaintext or an erb file.
+
       Further customization will require that you write your own task.
     DESC
     task :disable, :roles => :web, :except => { :no_release => true } do
       require 'erb'
-      on_rollback { run "rm #{shared_path}/system/#{maintenance_basename}.html" }
+      on_rollback { run "rm -f #{shared_path}/system/#{maintenance_basename}.html" }
 
       warn <<-EOHTACCESS
 
-        # Please add something like this to your site's htaccess to redirect users to the maintenance page.
+        # Please add something like this to your site's Apache htaccess to redirect users to the maintenance page.
         # More Info: http://www.shiftcommathree.com/articles/make-your-rails-maintenance-page-respond-with-a-503
 
         ErrorDocument 503 /system/#{maintenance_basename}.html
@@ -544,12 +559,23 @@ namespace :deploy do
         RewriteCond %{DOCUMENT_ROOT}/system/#{maintenance_basename}.html -f
         RewriteCond %{SCRIPT_FILENAME} !#{maintenance_basename}.html
         RewriteRule ^.*$  -  [redirect=503,last]
+
+        # Or if you are using Nginx add this to your server config:
+
+        if (-f $document_root/system/maintenance.html) {
+          return 503;
+        }
+        error_page 503 @maintenance;
+        location @maintenance {
+          rewrite  ^(.*)$  /system/maintenance.html last;
+          break;
+        }
       EOHTACCESS
 
       reason = ENV['REASON']
       deadline = ENV['UNTIL']
 
-      template = File.read(File.join(File.dirname(__FILE__), "templates", "maintenance.rhtml"))
+      template = File.read(maintenance_template_path)
       result = ERB.new(template).result(binding)
 
       put result, "#{shared_path}/system/#{maintenance_basename}.html", :mode => 0644
@@ -562,7 +588,7 @@ namespace :deploy do
       web-accessible again.
     DESC
     task :enable, :roles => :web, :except => { :no_release => true } do
-      run "rm #{shared_path}/system/#{maintenance_basename}.html"
+      run "rm -f #{shared_path}/system/#{maintenance_basename}.html"
     end
   end
 end
